@@ -77,7 +77,8 @@ static char *_hc08_keywords[] =
 
 void hc08_assignRegisters (ebbIndex *);
 
-static int regParmFlg = 0;      /* determine if we can register a parameter */
+static int regParmFlg;      /* determine if we can register a parameter */
+static struct sym_link *regParmFuncType;
 
 static void
 _hc08_init (void)
@@ -97,11 +98,15 @@ static void
 _hc08_reset_regparm (struct sym_link *funcType)
 {
   regParmFlg = 0;
+  regParmFuncType = funcType;
 }
 
 static int
 _hc08_regparm (sym_link * l, bool reentrant)
 {
+  if (IFFUNC_HASVARARGS (regParmFuncType))
+    return 0;
+
   int size = getSize(l);
 
   /* If they fit completely, the first two bytes of parameters can go */
@@ -134,20 +139,25 @@ _hc08_parseOptions (int *pargc, char **argv, int *i)
     {
       options.out_fmt = 'E';
       debugFile = &dwarf2DebugFile;
-      return TRUE;
+      return true;
     }
 
   if (!strcmp (argv[*i], "--oldralloc"))
     {
-      options.oldralloc = TRUE;
-      return TRUE;
+      options.oldralloc = true;
+      return true;
     }
 
-  return FALSE;
+  return false;
 }
 
+#define OPTION_SMALL_MODEL          "--model-small"
+#define OPTION_LARGE_MODEL          "--model-large"
+
 static OPTION _hc08_options[] =
   {
+    {0, OPTION_SMALL_MODEL, NULL, "8-bit address space for data"},
+    {0, OPTION_LARGE_MODEL, NULL, "16-bit address space for data (default)"},
     {0, "--out-fmt-elf", NULL, "Output executable in ELF format" },
     {0, "--oldralloc", NULL, "Use old register allocator"},
     {0, NULL }
@@ -310,7 +320,7 @@ _hc08_genIVT (struct dbuf_s * oBuf, symbol ** interrupts, int maxInterrupts)
     }
   dbuf_printf (oBuf, "\t.dw\t%s", "__sdcc_gs_init_startup\n");
 
-  return TRUE;
+  return true;
 }
 
 /* Generate code to copy XINIT to XISEG */
@@ -356,9 +366,9 @@ hasExtBitOp (int op, int size)
       || op == GETBYTE
       || op == GETWORD
      )
-    return TRUE;
+    return true;
   else
-    return FALSE;
+    return false;
 }
 
 /* Indicate the expense of an access to an output storage class */
@@ -397,6 +407,12 @@ hc08_dwarfRegNum (const struct reg_info *reg)
   return -1;
 }
 
+static bool
+_hasNativeMulFor (iCode *ic, sym_link *left, sym_link *right)
+{
+  return getSize (left) == 1 && getSize (right) == 1;
+}
+
 typedef struct asmLineNode
   {
     int size;
@@ -763,7 +779,7 @@ PORT hc08_port =
   NULL,                         /* Processor name */
   {
     glue,
-    FALSE,                      /* Emit glue around main */
+    false,                      /* Emit glue around main */
     MODEL_SMALL | MODEL_LARGE,
     MODEL_LARGE,
     NULL,                       /* model == target */
@@ -791,11 +807,11 @@ PORT hc08_port =
     hc08_getInstructionSize,
   },
   {
-        /* Sizes: char, short, int, long, long long, ptr, fptr, gptr, bit, float, max */
-    1, 2, 2, 4, 8, 2, 2, 2, 1, 4, 4
+    /* Sizes: char, short, int, long, long long, near ptr, far ptr, gptr, func ptr, banked func ptr, bit, float */
+    1, 2, 2, 4, 8, 2, 2, 2, 2, 0, 1, 4
   },
   /* tags for generic pointers */
-  { 0x00, 0x40, 0x60, 0x80 },           /* far, near, xstack, code */
+  { 0x00, 0x00, 0x00, 0x00 },           /* far, near, xstack, code */
   {
     "XSEG",
     "STACK",
@@ -825,17 +841,18 @@ PORT hc08_port =
   },
   { _hc08_genExtraAreas,
     NULL },
+  0,                      // ABI revision
   {
     -1,         /* direction (-1 = stack grows down) */
     0,          /* bank_overhead (switch between register banks) */
     4,          /* isr_overhead */
     2,          /* call_overhead */
     0,          /* reent_overhead */
-    0           /* banked_overhead (switch between code banks) */
+    0,          /* banked_overhead (switch between code banks) */
+    1           /* sp is offset by 1 from last item pushed */
   },
-    /* hc08 has an 8 bit mul */
   {
-    1, 5
+    5, false
   },
   {
     hc08_emitDebuggerSymbol,
@@ -879,18 +896,18 @@ PORT hc08_port =
   _hc08_regparm,
   NULL,                         /* process_pragma */
   NULL,                         /* getMangledFunctionName */
-  NULL,                         /* hasNativeMulFor */
+  _hasNativeMulFor,             /* hasNativeMulFor */
   hasExtBitOp,                  /* hasExtBitOp */
   oclsExpense,                  /* oclsExpense */
-  TRUE,                         /* use_dw_for_init */
-  FALSE,                        /* little_endian */
+  true,                         /* use_dw_for_init */
+  false,                        /* little_endian */
   0,                            /* leave lt */
   0,                            /* leave gt */
   1,                            /* transform <= to ! > */
   1,                            /* transform >= to ! < */
   1,                            /* transform != to !(a == b) */
   0,                            /* leave == */
-  FALSE,                        /* No array initializer support. */
+  false,                        /* No array initializer support. */
   cseCostEstimation,
   NULL,                         /* no builtin functions */
   GPOINTER,                     /* treat unqualified pointers as "generic" pointers */
@@ -908,7 +925,7 @@ PORT s08_port =
   NULL,                         /* Processor name */
   {
     glue,
-    FALSE,                      /* Emit glue around main */
+    false,                      /* Emit glue around main */
     MODEL_SMALL | MODEL_LARGE,
     MODEL_LARGE,
     NULL,                       /* model == target */
@@ -936,11 +953,11 @@ PORT s08_port =
     hc08_getInstructionSize,
   },
   {
-        /* Sizes: char, short, int, long, long long, ptr, fptr, gptr, bit, float, max */
-    1, 2, 2, 4, 8, 2, 2, 2, 1, 4, 4
+    /* Sizes: char, short, int, long, long long, near ptr, far ptr, gptr, func ptr, banked func ptr, bit, float */
+    1, 2, 2, 4, 8, 2, 2, 2, 2, 0, 1, 4
   },
   /* tags for generic pointers */
-  { 0x00, 0x40, 0x60, 0x80 },           /* far, near, xstack, code */
+  { 0x00, 0x00, 0x00, 0x00 },           /* far, near, xstack, code */
   {
     "XSEG",
     "STACK",
@@ -970,17 +987,18 @@ PORT s08_port =
   },
   { _hc08_genExtraAreas,
     NULL },
+  0,
   {
     -1,         /* direction (-1 = stack grows down) */
     0,          /* bank_overhead (switch between register banks) */
     4,          /* isr_overhead */
     2,          /* call_overhead */
     0,          /* reent_overhead */
-    0           /* banked_overhead (switch between code banks) */
+    0,          /* banked_overhead (switch between code banks) */
+    1           /* sp is offset by 1 from last item pushed */
   },
-    /* hc08 has an 8 bit mul */
   {
-    1, 5
+    5, false
   },
   {
     hc08_emitDebuggerSymbol,
@@ -1024,18 +1042,18 @@ PORT s08_port =
   _hc08_regparm,
   NULL,                         /* process_pragma */
   NULL,                         /* getMangledFunctionName */
-  NULL,                         /* hasNativeMulFor */
+  _hasNativeMulFor,             /* hasNativeMulFor */
   hasExtBitOp,                  /* hasExtBitOp */
   oclsExpense,                  /* oclsExpense */
-  TRUE,                         /* use_dw_for_init */
-  FALSE,                        /* little_endian */
+  true,                         /* use_dw_for_init */
+  false,                        /* little_endian */
   0,                            /* leave lt */
   0,                            /* leave gt */
   1,                            /* transform <= to ! > */
   1,                            /* transform >= to ! < */
   1,                            /* transform != to !(a == b) */
   0,                            /* leave == */
-  FALSE,                        /* No array initializer support. */
+  false,                        /* No array initializer support. */
   cseCostEstimation,
   NULL,                         /* no builtin functions */
   GPOINTER,                     /* treat unqualified pointers as "generic" pointers */
